const fs = require('fs');
const path = require('path');
const jsonDataPath = path.join(__dirname, 'data.json');
const jsonData = require(jsonDataPath);
const templateContent = generateTemplate(jsonData);
const styles = generateStyles(jsonData.template,[]);

const componentCode = `
<template>
  ${templateContent}
</template>

<script>
export default {
  ${generateScriptContent(jsonData.script)}
};
</script>

<style scoped>

${styles}

</style>
`;
const outputFilePath = path.join(__dirname, 'GeneratedComponent.vue');
fs.writeFileSync(outputFilePath, componentCode);
console.log(`Vue component generated successfully at ${outputFilePath}`);
function generateTemplate(data) {
  function generateTemplateRecursive(data) {
    if (typeof data === 'string') {
      return data;
    } else if (Array.isArray(data)) {
      return data.map(generateTemplateRecursive).join('\n');
    } else if (typeof data === 'object') {
      const tagName = data.tagName || 'div';
      const properties = data.properties
        ? ' ' + Object.entries(data.properties)
            .map(([key, value]) => `${key}="${value}"`)
            .join(' ')
        : '';
      const directives = Object.entries(data)
        .filter(([key]) => key.startsWith('v-'))
        .map(([key, value]) => `${key}="${value}"`);
      const directiveString = directives.length > 0 ? ` ${directives.join(' ')}` : '';
      const children = data.children ? generateTemplateRecursive(data.children) : '';
      return `<${tagName}${properties}${directiveString}>${children}</${tagName}>`;
    } else {
      return String(data);
    }
  }
  return `${generateTemplateRecursive(data.template)}`;
}
function generateScriptContent(scriptData) {
  console.log('scriptData:', scriptData); 
  const scriptProperties = scriptData.exportDefault ? generateDataProperties(scriptData.exportDefault) : '{}';
  return `
  data() {
    return ${scriptProperties}
  },
  `;
}
function generateDataProperties(data) {
  const properties = Object.entries(data)
    .map(([key, value]) => {
      if (typeof value === 'object' && !Array.isArray(value)) {
        return `${key}: ${generateDataProperties(value)}`;
      } else {
        return `${key}: ${JSON.stringify(value)}`;
      }
    })
    .join(', ');
  return `{${properties}}`;
}


function generateStyles(data) {
  function traverseStyles(data, parentSelector = '') {
    let styles = '';

    if (data.properties && data.properties.class && data.styles) {
      const selector = parentSelector ? `${parentSelector} ${data.tagName}` : data.tagName;
      const classList = data.properties.class.split(' ');

      styles += `${selector} { ${generateStyleProperties(data.styles)} }\n`;
    }

    if (data.children) {
      data.children.forEach(child => {
        styles += traverseStyles(child, parentSelector ? `${parentSelector} ${data.tagName}` : data.tagName);
      });
    }

    return styles;
  }

  function generateStyleProperties(styles) {
    return Object.entries(styles || {})
      .map(([property, value]) => `${property}: ${value};`)
      .join(' ');
  }

  return traverseStyles(data);
}


